diff --git a/contrib/auth_delay/Makefile b/contrib/auth_delay/Makefile
index 4b86ec37f0..b65097789a 100644
--- a/contrib/auth_delay/Makefile
+++ b/contrib/auth_delay/Makefile
@@ -3,6 +3,8 @@
 MODULES = auth_delay
 PGFILEDESC = "auth_delay - delay authentication failure reports"
 
+TAP_TESTS = 1
+
 ifdef USE_PGXS
 PG_CONFIG = pg_config
 PGXS := $(shell $(PG_CONFIG) --pgxs)
diff --git a/contrib/auth_delay/t/001_auth_delay.pl b/contrib/auth_delay/t/001_auth_delay.pl
new file mode 100644
index 0000000000..644026e4f2
--- /dev/null
+++ b/contrib/auth_delay/t/001_auth_delay.pl
@@ -0,0 +1,87 @@
+
+# Copyright (c) 2021-2022, PostgreSQL Global Development Group
+
+use strict;
+use warnings;
+
+use PostgreSQL::Test::Cluster;
+use PostgreSQL::Test::Utils;
+use Test::More;
+use Time::HiRes qw(gettimeofday tv_interval);
+
+# Delete pg_hba.conf from the given node, add a new entry to it
+# and then execute a reload to refresh it.
+sub reset_pg_hba
+{
+    my $node       = shift;
+    my $hba_method = shift;
+
+    unlink($node->data_dir . '/pg_hba.conf');
+    # just for testing purposes, use a continuation line
+    $node->append_conf('pg_hba.conf', "local all all $hba_method");
+    $node->reload;
+    return;
+}
+
+# Test access for a single role, with password
+sub test_login
+{
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
+
+    my $node          = shift;
+    my $role          = shift;
+    my $password      = shift;
+    my $expected_res  = shift;
+    my $status_string = 'failed';
+
+    $status_string = 'success' if ($expected_res eq 0);
+
+    my $connstr = "user=$role";
+    my $testname =
+    "authentication $status_string for role $role with password $password";
+
+    $ENV{"PGPASSWORD"} = $password;
+
+    if ($expected_res eq 0)
+    {
+        $node->connect_ok($connstr, $testname);
+    }
+    else
+    {
+        # No checks of the error message, only the status code.
+        $node->connect_fails($connstr, $testname);
+    }
+}
+
+note "setting up PostgreSQL instance";
+
+my $delay_milliseconds = 500;
+
+my $node = PostgreSQL::Test::Cluster->new('main');
+$node->init;
+$node->append_conf('postgresql.conf',
+    qq{shared_preload_libraries = 'auth_delay'
+       auth_delay.milliseconds  = '$delay_milliseconds'});
+$node->start;
+
+$node->safe_psql(
+    'postgres',
+    "CREATE ROLE user_role LOGIN PASSWORD 'pass';");
+reset_pg_hba($node, 'password');
+
+note "running tests";
+
+# check enter wrong password
+my $t0 = [gettimeofday];
+test_login($node, 'user_role', "wrongpass", 2);
+my $elapsed = tv_interval($t0, [gettimeofday]);
+ok($elapsed >= $delay_milliseconds / 1000, "auth_delay $elapsed seconds");
+
+# check enter correct password
+my $t0 = [gettimeofday];
+test_login($node, 'user_role', "pass", 0);
+my $elapsed = tv_interval($t0, [gettimeofday]);
+ok($elapsed < $delay_milliseconds / 1000, "auth_delay $elapsed seconds");
+
+done_testing();
+
